==> afnix_2.5.1.orig.tar.gz: afnix-src-2.5.1/src/srv/tls/shl/TlsUtils.cpp <==
// ---------------------------------------------------------------------------
// - TlsUtils.cpp                                                            -
// - afnix:tls service - tls utility class implementation                    -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2015 amaury darsch                                   -
// ---------------------------------------------------------------------------

#include "Time.hpp"
#include "System.hpp"
#include "Utility.hpp"
#include "TlsUtils.hpp"
#include "Exception.hpp"

namespace afnix {

  // create a random array by size and time flag

  t_byte* TlsUtils::random (const long size, const bool tflg) {
    // check the array size
    if (size < 0L) {
      throw Exception ("tls-error", "invalid random array size");
    }
    // check for nil
    if (size == 0L) return nilp;
    // allocate the array and fill it
    t_byte* result = new t_byte[size];
    for (long k = 0; k < size; k++) result[k] = Utility::byternd ();
    // eventually set the unix time in the first 4 bytes
    if (tflg == true) {
      union {
	t_quad tunx;
	t_byte tval[4];
      };
      tunx = System::qswap (Time::getuclk ());
      result[0] = tval[0];
      result[1] = tval[1];
      result[2] = tval[2];
      result[3] = tval[3];
    }
    return result;
  }
}

==> afnix_2.6.3.orig.tar.gz: afnix-src-2.6.3/src/srv/tls/shl/TlsUtils.cpp <==
// ---------------------------------------------------------------------------
// - TlsUtils.cpp                                                            -
// - afnix:tls service - tls utility class implementation                    -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2016 amaury darsch                                   -
// ---------------------------------------------------------------------------

#include "Time.hpp"
#include "Ascii.hpp"
#include "System.hpp"
#include "Unicode.hpp"
#include "Utility.hpp"
#include "TlsUtils.hpp"
#include "Exception.hpp"

namespace afnix {

  // create a random array by size and time flag

  t_byte* TlsUtils::random (const long size, const bool tflg) {
    // check the array size
    if (size < 0L) {
      throw Exception ("tls-error", "invalid random array size");
    }
    // check for nil
    if (size == 0L) return nilp;
    // allocate the array and fill it
    t_byte* result = new t_byte[size];
    for (long k = 0; k < size; k++) result[k] = Utility::byternd ();
    // eventually set the unix time in the first 4 bytes
    if (tflg == true) {
      union {
	t_quad tunx;
	t_byte tval[4];
      };
      tunx = System::qswap (Time::getuclk ());
      result[0] = tval[0];
      result[1] = tval[1];
      result[2] = tval[2];
      result[3] = tval[3];
    }
    return result;
  }

  // the standard p_hash function of the tls

  Buffer TlsUtils::phash (Mac* hmac, const Buffer& seed, const long rlen) {
    // check for valid arguments
    if ((hmac == nilp) || (seed.empty () == true) || (rlen <= 0L)) {
      throw Exception ("tls-error", "invalid arguments with p_hash");
    }
    // preset seed buffer
    long    slen = seed.length ();
    t_byte* sbuf = (t_byte*) seed.tochar ();
    try {
      // precompute A(1) = hmac (secret, seed)
      Buffer hbuf;
      hmac->reset   ();
      hmac->process (sbuf, slen);
      hmac->finish  ();
      hmac->pushb   (hbuf);
      // loop until enough bytes
      Buffer rbuf;
      while (rbuf.length () < rlen) {
	// save the buffer
	Buffer ibuf = hbuf;
	// one hash loop
	hmac->reset   ();
	hmac->process (hbuf);
	hmac->process (sbuf, slen);
	hmac->finish  ();
	hmac->pushb   (rbuf);
	// compute A1
	hmac->reset   ();
	hmac->process (ibuf);
	hmac->finish  ();
	hbuf.reset    ();
	hmac->pushb   (hbuf);
      }
      // trim extra bytes and lock
      rbuf.trim (rlen, false);
      delete [] sbuf;
      return rbuf;
    } catch (...) {
      delete [] sbuf;
      throw;
    }
  }
}
